#
# interface.py
# User interface class(es) for repoman
#
# Copyright (C) 2006-2007  Red Hat, Inc. All rights reserved.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2.  This program is distributed in the
# hope that it will be useful, but WITHOUT ANY WARRANTY expressed or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
# details.  You should have received a copy of the GNU General Public
# License along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
# USA. Any Red Hat trademarks that are incorporated in the source code or
# documentation are not subject to the GNU General Public License and may
# only be used or replicated with the express permission of Red Hat, Inc.
#
# Red Hat Author: David Cantrell <dcantrell@redhat.com>
#                 Chris Lumens <clumens@redhat.com>
#

import os
import gtk
import gtk.glade
import gobject

from rhpl.translate import _, N_
import rhpl.translate as translate
domain = 'repoman'
translate.textdomain(domain)
gtk.glade.bindtextdomain(domain)

from repoman.repos import *
from repoman.repofile import *
from repoman.repostanza import *

# Filter out all debuginfo and source repo stanzas from the UI.
def filterFunc (model, iter):
	# When we append to the ListStore after a filter has already been added,
	# GTK first adds an empty row and then populates it with data.  But as
	# soon as the empty row is appended, a signal gets emitted and this
	# function will be called.  The result is tracebacks, so ignore those
	# empty rows for now.
	if model.get_value(iter, 3) is None:
		return False

	name = model.get_value(iter, 1)
	return name.find("-debuginfo") == -1 and name.find("-source") == -1

class TrackingPage:
	def __init__(self, interface):
		self.intf = interface
		self.xml = self.intf.xml
		self.repos = self.intf.repos

		self.radioStable = self.xml.get_widget("stable")
		self.radioDevelopment = self.xml.get_widget("development")
		self.checkUpdates = self.xml.get_widget("updates")
		self.checkTesting = self.xml.get_widget("updates_testing")

		self.radioStable.connect("toggled", self.radioStable_changed)
		self.checkUpdates.connect("toggled", self.checkUpdates_changed)
		self.checkTesting.connect("toggled", self.checkTesting_changed)

		# Initialize UI based on what repos are selected.
		try:
			if self.repos.getStanzaByName("updates").isEnabled():
				self.checkUpdates.set_active(True)
		except:
			self.disableTracking()

		try:
			if self.repos.getStanzaByName("updates-testing").isEnabled():
				self.checkTesting.set_active(True)
		except:
			self.disableTracking()

		try:
			if self.repos.getStanzaByName("core").isEnabled():
				self.radioStable.set_active(True)
			else:
				self.radioDevelopment.set_active(True)
		except:
			self.disableTracking()

	# Disable tracking page options
	def disableTracking(self):
		self.radioStable.set_sensitive(False)
		self.radioDevelopment.set_sensitive(False)
		self.checkUpdates.set_sensitive(False)
		self.checkTesting.set_sensitive(False)

	# Check button callbacks
	def checkUpdates_changed(self, args):
		if self.checkUpdates.get_active():
			self.repos.getStanzaByName("updates").enable()
			self.intf.repositoriesPage.checkRepo("updates", True)
		else:
			self.repos.getStanzaByName("updates").disable()
			self.intf.repositoriesPage.checkRepo("updates", False)

	def checkTesting_changed(self, args):
		if self.checkTesting.get_active():
			self.repos.getStanzaByName("updates-testing").enable()
			self.intf.repositoriesPage.checkRepo("updates-testing", True)
		else:
			self.repos.getStanzaByName("updates-testing").disable()
			self.intf.repositoriesPage.checkRepo("updates-testing", False)

	# Radio group callbacks
	def radioStable_changed(self, args):
		disableRepos = []
		enableRepos = []
		updatesSensitivity = False

		if self.radioStable.get_active():
			disableRepos = ["development", "extras-development"]
			enableRepos = ["core", "extras"]
			updatesSensitivity = True

			if self.checkUpdates.get_active():
				enableRepos.append("updates")
			if self.checkTesting.get_active():
				enableRepos.append("updates-testing")
		else:
			disableRepos = ["core", "extras", "updates", "updates-testing"]
			enableRepos = ["development", "extras-development"]
			updatesSensitivity = False

		for repoName in disableRepos:
			self.repos.getStanzaByName(repoName).disable()
			self.intf.repositoriesPage.checkRepo(repoName, False)

		for repoName in enableRepos:
			self.repos.getStanzaByName(repoName).enable()
			self.intf.repositoriesPage.checkRepo(repoName, True)

		self.checkUpdates.set_sensitive(updatesSensitivity)
		self.checkTesting.set_sensitive(updatesSensitivity)

class RepositoriesPage:
	def __init__(self, interface):
		self.intf = interface
		self.xml = self.intf.xml
		self.repos = self.intf.repos

		self.repo_tree = self.xml.get_widget("repo_tree")
		self.repo_tree.connect("row_activated", self.repo_tree_double_click)

		self.addRepoButton = self.xml.get_widget("addRepoButton")
		self.removeRepoButton = self.xml.get_widget("removeRepoButton")
		self.editRepoButton = self.xml.get_widget("editRepoButton")

		self.addRepoButton.connect("clicked", self.addClicked)
		self.removeRepoButton.connect("clicked", self.removeClicked)
		self.editRepoButton.connect("clicked", self.editClicked)

		# build the repo tree
		self.repo_tree_store = gtk.ListStore(gobject.TYPE_BOOLEAN,
		                                     gobject.TYPE_STRING,
		                                     gobject.TYPE_STRING,
		                                     gobject.TYPE_PYOBJECT)
		self.repo_tree.set_property("headers-visible", True)

		checkboxrenderer = gtk.CellRendererToggle()
		checkboxrenderer.connect("toggled", self.repoCheckboxToggled)

		col = gtk.TreeViewColumn(_("Enabled"), checkboxrenderer, active=0)
		self.repo_tree.append_column(col)

		col = gtk.TreeViewColumn(_("Name"), gtk.CellRendererText(), text = 1)
		self.repo_tree.append_column(col)

		col = gtk.TreeViewColumn(_("Description"), gtk.CellRendererText(), text = 2)
		self.repo_tree.append_column(col)

		# add a row for each repo stanza with
		# 1) a checkbox for enabling/disabling it
		# 2) the name of the repository
		# 3) the description of the repository
		# 4) the stanza object itself
		for f in self.repos.getRepoFiles():
			stanzas = f.getRepoStanzas()
			for key in stanzas.keys():
				s = stanzas[key]
				self.repo_tree_store.append([s.isEnabled(), s.getName(), s.getDescription(), s])

		self.repo_tree_sort = gtk.TreeModelSort(self.repo_tree_store)
		self.repo_tree_sort.set_sort_column_id(1, gtk.SORT_ASCENDING)
		self.repo_tree_filter = self.repo_tree_sort.filter_new()
		self.repo_tree_filter.set_visible_func(filterFunc)
		self.repo_tree.set_model(self.repo_tree_filter)

		# Instantiate the dialogs, since we're just going to keep one around
		# and keep showing/filling in/hiding during the life of the program.
		self.editDialog = EditRepoDialog(self.xml, self.repos)
		self.addDialog = AddRepoDialog(self.xml, self.repos, self.repo_tree_store)

	# Given a row in the sorted model, return everything needed to update
	# the state.
	def _findListStoreRow(self, row):
		# row is the number of the row clicked on the UI, which is an index
		# into the TreeModelFilter, and not the actual store.
		filteredModel = self.repo_tree.get_model()
		filteredIter = filteredModel.get_iter(row)

		# Now that we have the iter set in the filtered model, convert it into
		# an iter on the actual store and return that.
		sortedModel = filteredModel.get_model()
		listModel = sortedModel.get_model()
		sortedIter = filteredModel.convert_iter_to_child_iter(filteredIter)
		listIter = sortedModel.convert_iter_to_child_iter(None, sortedIter)
		stanza = listModel.get_value(listIter, 3)

		return (listModel, listIter, stanza)

	# Check/unchcek the given repoName in the interface.
	def checkRepo(self, repoName, isChecked):
		iter = self.repo_tree_store.get_iter_first()

		while iter:
			if self.repo_tree_store.get_value(iter, 1) == repoName:
				self.repo_tree_store.set_value(iter, 0, isChecked)
				return

			iter = self.repo_tree_store.iter_next(iter)

	# Button callbacks
	def addClicked(self, args):
		self.addDialog.run()

	def removeClicked(self, args):
		iter = self.repo_tree.get_selection().get_selected()[1]

		if iter is not None:
			stanzaObj = self.repo_tree.get_model().get_value(iter, 3)
			name = stanzaObj.getName()

			msg = _("Do you really want to delete the repository named %s?") % name
			dialog = gtk.MessageDialog(parent=self.intf.toplevel,
			                           flags=0,
			                           type=gtk.MESSAGE_WARNING,
			                           buttons=gtk.BUTTONS_YES_NO,
			                           message_format=msg)
			dialog.set_title(_("Delete Repository?"))
			dialog.set_position(gtk.WIN_POS_CENTER)
			rc = dialog.run()

			if rc == gtk.RESPONSE_YES:
				for file in self.repos.getRepoFiles():
					try:
						file.deleteStanzaByName(name)

						# More iter games.
						filter_iter = self.repo_tree_filter.convert_iter_to_child_iter(iter)
						sort_iter = self.repo_tree_sort.convert_iter_to_child_iter(None, filter_iter)
						self.repo_tree_store.remove(sort_iter)
					except:
						pass

			dialog.destroy()

	def editClicked(self, args):
		iter = self.repo_tree.get_selection().get_selected()[1]

		if iter is not None:
			stanzaObj = self.repo_tree.get_model().get_value(iter, 3) 
			self.editDialog.setStanzaObj(stanzaObj)
			self.editDialog.fillInUI()
			self.editDialog.run()

	# repo_tree callbacks
	def repo_tree_double_click(self, view, path, col, *args):
		(model, iter, stanzaObj) = self._findListStoreRow(int(path[0]))
		self.editDialog.setStanzaObj(stanzaObj)
		self.editDialog.fillInUI()
		self.editDialog.run()

	def repoCheckboxToggled(self, data, row):
		(model, iter, stanza) = self._findListStoreRow(int(row))
		is_checked = stanza.isEnabled()

		# Update the checkbox in the UI.
		model.set_value(iter, 0, not is_checked)

		# Also need to update the RepoStanza object itself.
		if is_checked:
			stanza.disable()
		else:
			stanza.enable()

		# Finally, we might need to update the checkboxes on the first
		# page of the notebook.
		repoName = model.get_value(iter, 1)

		if repoName == "updates":
			self.intf.trackingPage.checkUpdates.set_active(stanza.enabled)
		elif repoName == "updates-testing":
			self.intf.trackingPage.checkTesting.set_active(stanza.enabled)

# Base class for AddRepoDialog and EditRepoDialog.
class RepoDialog:
	def __init__(self, xml, repos):
		self.xml = xml
		self.repos = repos

		self.dialog = None
		self.radioBaseURL = None
		self.entryBaseURL = None
		self.radioMirrorList = None
		self.entryMirrorList = None

	def errorMsg(self, msg):
		dlg = gtk.MessageDialog(None, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK,
				                msg)
		dlg.set_position(gtk.WIN_POS_CENTER)
		dlg.set_modal(True)
		dlg.run()
		dlg.destroy()

	def run(self):
		while True:
			rc = self.dialog.run()
			if rc != gtk.RESPONSE_NONE:
				break

		self.dialog.hide()

	# Button callbacks.
	def destroy(self, args):
		self.dialog.hide()

	# UI helpers
	def _setBaseURLStatus(self, status):
		self.radioBaseURL.set_active(status)
		self.entryBaseURL.set_sensitive(status)
		self.radioMirrorList.set_active(not status)
		self.entryMirrorList.set_sensitive(not status)

	# Radio group callbacks
	def radioBaseURL_changed(self, args):
		self._setBaseURLStatus(self.radioBaseURL.get_active())

class AddRepoDialog(RepoDialog):
	def __init__(self, xml, repos, tree_store):
		RepoDialog.__init__(self, xml, repos)
		self.tree_store = tree_store

		self.dialog = self.xml.get_widget("addRepoDialog")
		self.editName = self.xml.get_widget("addEditName")
		self.editDescription = self.xml.get_widget("addEditDescription")

		self.cancelButton = self.xml.get_widget("addCancelButton")
		self.okButton = self.xml.get_widget("addOkButton")

		self.radioBaseURL = self.xml.get_widget("addRadioBaseURL")
		self.radioMirrorList = self.xml.get_widget("addRadioMirrorList")
		self.entryBaseURL = self.xml.get_widget("addEntryBaseURL")
		self.entryMirrorList = self.xml.get_widget("addEntryMirrorList")

		self.checkEnabled = self.xml.get_widget("addCheckEnabled")
		self.checkVerify = self.xml.get_widget("addCheckVerify")

		self.okButton.connect("clicked", self.okClicked)
		self.cancelButton.connect("clicked", self.destroy)
		self.dialog.connect("destroy", self.destroy)

		self.radioBaseURL.connect("toggled", self.radioBaseURL_changed)

	def run(self):
		self.editName.set_text("")
		self.editDescription.set_text("")
		self.entryBaseURL.set_text("")
		self.entryMirrorList.set_text("")
		self._setBaseURLStatus(True)
		self.checkEnabled.set_active(False)
		self.checkVerify.set_active(False)
		RepoDialog.run(self)

	# Button callbacks
	def okClicked(self, args):
		newStanza = RepoStanza()

		if self.editName.get_text().strip() == "":
			self.errorMsg(_("Please enter a name for the new repository."))
			self.dialog.response(gtk.RESPONSE_NONE)
			return
		else:
			newStanza.setName(self.editName.get_text())
			newStanza.setDescription(self.editDescription.get_text())

		if self.radioBaseURL.get_active():
			if self.entryBaseURL.get_text().strip() == "":
				self.errorMsg(_("Please enter a URL for the new repository."))
				self.dialog.response(gtk.RESPONSE_NONE)
				return
			else:
				newStanza.enableBaseURL()
		else:
			if self.entryMirrorList.get_text().strip() == "":
				self.errorMsg(_("Please enter a URL to the mirror list for the new repository."))
				self.dialog.response(gtk.RESPONSE_NONE)
				return
			else:
				newStanza.enableMirrorList()

		newStanza.setBaseURL(self.entryBaseURL.get_text())
		newStanza.setMirrorList(self.entryMirrorList.get_text())

		if self.checkEnabled.get_active():
			newStanza.enable()
		else:
			newStanza.disable()

		if self.checkVerify.get_active():
			newStanza.enableVerify()
		else:
			newStanza.disableVerify()

		# Make sure no stanza already exists with this name.
		oldStanza = self.repos.getStanzaByName(newStanza.getName())
		if oldStanza and not oldStanza.isRemoved():
			self.errorMsg(_("A repository by that name already exists."))
			self.dialog.response(gtk.RESPONSE_NONE)
			return

		self.tree_store.append([newStanza.isEnabled(), newStanza.getName(), newStanza.getDescription(), newStanza])

		for file in self.repos.getRepoFiles():
			# If we find the repoman.repo file, add to it.  Otherwise,
			# create a new file.
			if file.getRepoFileName().endswith("repoman.repo"):
				file.stanzas[newStanza.getName()] = newStanza
				return

		newFile = RepoFile(self.repos.repodir + "/repoman.repo")
		newFile.stanzas[newStanza.getName()] = newStanza
		self.repos.repofiles.append(newFile)

class EditRepoDialog(RepoDialog):
	def __init__(self, xml, repos):
		RepoDialog.__init__(self, xml, repos)
		self.stanzaObj = None

		self.dialog = self.xml.get_widget("editRepoDialog")
		self.editName = self.xml.get_widget("editNameLabel")
		self.editDescription = self.xml.get_widget("editDescriptionLabel")

		self.cancelButton = self.xml.get_widget("editCancelButton")
		self.okButton = self.xml.get_widget("editOkButton")

		self.radioBaseURL = self.xml.get_widget("editRadioBaseURL")
		self.radioMirrorList = self.xml.get_widget("editRadioMirrorList")
		self.entryBaseURL = self.xml.get_widget("editEntryBaseURL")
		self.entryMirrorList = self.xml.get_widget("editEntryMirrorList")

		self.checkEnabled = self.xml.get_widget("editCheckEnabled")
		self.checkVerify = self.xml.get_widget("editCheckVerify")

		self.okButton.connect("clicked", self.okClicked)
		self.cancelButton.connect("clicked", self.destroy)
		self.dialog.connect("destroy", self.destroy)

		self.radioBaseURL.connect("toggled", self.radioBaseURL_changed)

	def setStanzaObj(self, stanzaObj):
		self.stanzaObj = stanzaObj

	def fillInUI(self):
		if not self.stanzaObj:
			return

		if self.stanzaObj.getBaseURL() is not None:
			self.entryBaseURL.set_text(self.stanzaObj.getBaseURL())
		else:
			self.entryBaseURL.set_text("")

		if self.stanzaObj.getMirrorList() is not None:
			self.entryMirrorList.set_text(self.stanzaObj.getMirrorList())
		else:
			self.entryMirrorList.set_text("")

		self._setBaseURLStatus(self.stanzaObj.isBaseURLEnabled())
		self.checkVerify.set_active(self.stanzaObj.isVerifyEnabled())

		self.checkEnabled.set_active(self.stanzaObj.isEnabled())

		self.editName.set_text(self.stanzaObj.getName())
		self.editDescription.set_text(self.stanzaObj.getDescription())

	def okClicked(self, args):
		# Use the current UI state to set values on the repo stanza object.
		if self.radioBaseURL.get_active():
			if self.entryBaseURL.get_text().strip() == "":
				self.errorMsg(_("Please enter a URL for the new repository."))
				self.dialog.response(gtk.RESPONSE_NONE)
				return
			else:
				self.stanzaObj.enableBaseURL()
		else:
			if self.entryMirrorList.get_text().strip() == "":
				self.errorMsg(_("Please enter a URL to the mirror list for the new repository."))
				self.dialog.response(gtk.RESPONSE_NONE)
				return
			else:
				self.stanzaObj.enableMirrorList()

		self.stanzaObj.setBaseURL(self.entryBaseURL.get_text())
		self.stanzaObj.setMirrorList(self.entryMirrorList.get_text())
		self.stanzaObj.verify = self.checkVerify.get_active()

class RepoMan:
	def __init__(self):
		self.repos = Repos()

		if os.access('repoman.glade', os.R_OK):
			self.xml = gtk.glade.XML('repoman.glade', domain=domain)
		else:
			self.xml = gtk.glade.XML('/usr/share/repoman/ui/repoman.glade', domain=domain)

		self.toplevel = self.xml.get_widget("main_window")
		self.mainOkButton = self.xml.get_widget("mainOkButton")
		self.mainCancelButton = self.xml.get_widget("mainCancelButton")
		self.mainApplyButton = self.xml.get_widget("mainApplyButton")

		self.toplevel.connect("destroy", self.destroy)
		self.mainOkButton.connect("clicked", self.okClicked)
		self.mainCancelButton.connect("clicked", self.destroy)
		self.mainApplyButton.connect("clicked", self.applyClicked)

		# Control for the second page of the UI, which allows manipulating
		# individual repositories.
		self.repositoriesPage = RepositoriesPage(self)

		# Control for the first page of the UI, which allows the high level
		# customization of what's enabled and disabled.
		self.trackingPage = TrackingPage(self)

	def writeRepos(self):
		try:
			self.repos.write()
		except RepoFileWriteFailure, e:
			msg = _("Failed writing new %s repository file.  Make sure you have the correct permissions to write repository files.  That usually means having superuser permissions on the system.") % (e.getRepoFileName(),)

			dialog = gtk.MessageDialog(parent=self.toplevel,
			                           flags=0,
			                           type=gtk.MESSAGE_WARNING,
			                           buttons=gtk.BUTTONS_OK,
			                           message_format=msg)
			dialog.set_title(_("Repository File Write Failure"))
			dialog.set_position(gtk.WIN_POS_CENTER)
			dialog.run()
			dialog.destroy()

	# Button callbacks
	def destroy(self, args):
		gtk.main_quit()

	def okClicked(self, args):
		self.writeRepos()
		gtk.main_quit()

	def applyClicked(self, args):
		self.writeRepos()
