#
# repofile.py
# Python class defining an /etc/yum.repos.d/*.repo file
#
# Copyright (C) 2006  Red Hat, Inc. All rights reserved.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2.  This program is distributed in the
# hope that it will be useful, but WITHOUT ANY WARRANTY expressed or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
# details.  You should have received a copy of the GNU General Public
# License along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
# USA. Any Red Hat trademarks that are incorporated in the source code or
# documentation are not subject to the GNU General Public License and may
# only be used or replicated with the express permission of Red Hat, Inc.
#
# Red Hat Author: David Cantrell <dcantrell@redhat.com>
#

import os
import stat
from tempfile import mkstemp
from repoman.repostanza import RepoStanza

class RepoFileWriteFailure(Exception):
	def __init__(self, repofile):
		self.repofile = repofile

	def getRepoFileName(self):
		return self.repofile

class RepoFile:
	def __init__(self, repofile):
		self.stanzas = {}
		self.repofile = repofile

		for stanza in self.parseStanzas():
			self.stanzas[stanza.getName()] = stanza

	def getRepoFileName(self):
		return self.repofile

	def setRepoFileName(self, newrepofile):
		self.repofile = newrepofile

	def getRepoStanzas(self):
		return self.stanzas

	def getStanzaByName(self, name):
		return self.stanzas[name]

	def deleteStanzaByName(self, name):
		self.stanzas[name].remove()

	def getNewStanzas(self, existing):
		new = self.stanzas.keys()

		for stanzaname in existing:
			new.remove(stanzaname)

		return new

	def read(self):
		if not os.access(self.repofile, os.R_OK):
			return None

		f = open(self.repofile, 'r')
		# remove '\n' from end of line (FIXME for windows)
		lines = map(lambda s: s[:-1],f.readlines())
		f.close()

		return lines

	def parseStanzas(self):
		stanzalist = []
		stanzalines = []

		lines = self.read()
		if lines is None:
			return stanzalist

		for l in lines:
			if l == '':
				continue

			if l[0] == '[' and l[-1] == ']':
				if stanzalines:
					s = RepoStanza(stanzalines)

					# a name of None means parse failure
					if s.getName() is not None:
						stanzalist.append(s)

				stanzalines = []
				stanzalines.append(l)
			else:
				if stanzalines:
					stanzalines.append(l)

		if stanzalines is not None:
			s = RepoStanza(stanzalines)

			# a name of None means parse failure
			if s.getName() is not None:
				stanzalist.append(s)

		return stanzalist

	def write(self):
		existing = []
		stanzalines = []
		lines = self.read()

		if lines is None:
			lines = []

		(fd, tmpfilename) = mkstemp(text=True)
		tmpfile = open(tmpfilename, 'w')

		# It's possible that a user will go and edit a repo file
		# while repoman is running.  If this happens, there's
		# nothing we can do about it.  The user shouldn't do that,
		# so to prevent reverting changes on a write() call, we
		# get the latest copy of the stanzalines and then replace
		# values as we need to.
		#
		# NOTE:  That's why we get _another_ copy of stanzalines
		# here rather than storing it when repoman is first run.
		firststanza = True
		for l in lines:
			if l == '':
				continue

			if l[0] == '[' and l[-1] == ']':
				if stanzalines is not None:
					tmpstanza = RepoStanza(stanzalines)
					if tmpstanza.getName() is None:
						stanzalines = []
						stanzalines.append(l)
						continue

				try:
					ns = self.getStanzaByName(tmpstanza.getName())
					ns.write(stanzalines, tmpfile)
					existing.append(ns.getName())
				except:
					tmpstanza.write(stanzalines, tmpfile)

				if firststanza:
					firststanza = False
				else:
					tmpfile.write("\n")

				stanzalines = []
				stanzalines.append(l)
			else:
				if stanzalines is not None:
					stanzalines.append(l)

		if stanzalines:
			tmpstanza = RepoStanza(stanzalines)
			if tmpstanza.getName() is not None:
				try:
					ns = self.getStanzaByName(tmpstanza.getName())
					ns.write(stanzalines, tmpfile)
					existing.append(ns.getName())
				except:
					tmpstanza.write(stanzalines, tmpfile)

		# handle any new repositories added by the user
		if not existing == []:
			tmpfile.write("\n")
			firststanza = True

		for newstanza in self.getNewStanzas(existing):
			ns = self.getStanzaByName(newstanza)
			ns.write([], tmpfile)

			if firststanza:
				firststanza = False
			else:
				tmpfile.write("\n")

		tmpfile.close()

		try:
			if os.access(self.getRepoFileName(), os.F_OK):
				statinfo = os.stat(self.getRepoFileName())
				perms = statinfo[0]
				uid = statinfo[4]
				gid = statinfo[5]
			else:
				perms = stat.S_IRUSR|stat.S_IWUSR|stat.S_IRGRP|stat.S_IROTH
				uid = 0
				gid = 0

			os.rename(tmpfilename, self.getRepoFileName())
			os.chmod(self.getRepoFileName(), perms)
			os.chown(self.getRepoFileName(), uid, gid)

			# remove file if it's zero length
			size = os.stat(self.getRepoFileName())
			if stat.ST_SIZE(size) == 0:
				try:
					os.unlink(self.getRepoFileName())
				except:
					pass
		except:
			try:
				os.unlink(tmpfilename)
			except:
				pass

			raise RepoFileWriteFailure(self.getRepoFileName())
