#
# repostanza.py
# Python class defining a repository stanza inside a *.repo file
#
# Copyright (C) 2006  Red Hat, Inc. All rights reserved.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2.  This program is distributed in the
# hope that it will be useful, but WITHOUT ANY WARRANTY expressed or
# implied, including the implied warranties of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
# details.  You should have received a copy of the GNU General Public
# License along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
# USA. Any Red Hat trademarks that are incorporated in the source code or
# documentation are not subject to the GNU General Public License and may
# only be used or replicated with the express permission of Red Hat, Inc.
#
# Red Hat Author: David Cantrell <dcantrell@redhat.com>
#

from yum.parser import varReplace
from yum.config import _getsysver
from rpmUtils.arch import getBaseArch, getCanonArch

substDict = {"basearch": getBaseArch(),
             "canonarch": getCanonArch(),
             "releasever": _getsysver("/", "fedora-release")}

class RepoStanza:
	def __init__(self, stanzalines=[]):
		self.name = None
		self.enabled = False
		self.description = None
		self.baseurl = None
		self.mirrorlist = None
		self.verify = False
		self.baseurlenabled = True
		self.mirrorlistenabled = False
		self.dirty = False
		self.removed = False

		for l in stanzalines:
			if l[0] == '[' and l[-1] == ']':
				self.name = l[1:-1]
			elif l[0:5] == 'name=':
				self.description = varReplace(l[5:], substDict)
			elif l[0:8] == 'enabled=':
				if l[-1] == '1':
					self.enabled = True
				elif l[-1] == '0':
					self.enabled = False
			elif l[0:9] == 'gpgcheck=':
				if l[-1] == '1':
					self.verify = True
				elif l[-1] == '0':
					self.verify = False
			elif l[0:8] == 'baseurl=':
				self.baseurl = l[8:]
				self.baseurlenabled = True
				self.mirrorlistenabled = False
			elif l[0:9] == '#baseurl=':
				self.baseurl = l[9:]
				self.baseurlenabled = False
				self.mirrorlistenabled = True
			elif l[0:11] == 'mirrorlist=':
				self.mirrorlist = l[11:]
				self.mirrorlistenabled = True
				self.baseurlenabled = False
			elif l[0:12] == '#mirrorlist=':
				self.mirrorlist = l[12:]
				self.mirrorlistenabled = False
				self.baseurlenabled = True

	def getName(self):
		return self.name

	def setName(self, newname):
		self.name = newname

	def isEnabled(self):
		return self.enabled

	def enable(self):
		self.enabled = True
		self.dirty = True

	def disable(self):
		self.enabled = False
		self.dirty = True

	def remove(self):
		self.removed = True

	def isRemoved(self):
		return self.removed

	def getDescription(self):
		return self.description

	def setDescription(self, newdescription):
		self.description = newdescription
		self.dirty = True

	def getBaseURL(self):
		return self.baseurl

	def setBaseURL(self, newbaseurl):
		self.baseurl = newbaseurl
		self.dirty = True

	def getMirrorList(self):
		return self.mirrorlist

	def setMirrorList(self, newmirrorlist):
		self.mirrorlist = newmirrorlist
		self.dirty = True

	def isVerifyEnabled(self):
		return self.verify

	def enableVerify(self):
		self.verify = True
		self.dirty = True

	def disableVerify(self):
		self.verify = False
		self.dirty = True

	def isBaseURLEnabled(self):
		return self.baseurlenabled

	def enableBaseURL(self):
		self.baseurlenabled = True
		self.mirrorlistenabled = False
		self.dirty = True

	def disableBaseURL(self):
		self.baseurlenabled = False
		self.mirrorlistenabled = True
		self.dirty = True

	def isMirrorListEnabled(self):
		return self.mirrorlistenabled

	def enableMirrorList(self):
		self.baseurlenabled = False
		self.mirrorlistenabled = True
		self.dirty = True

	def disableMirrorList(self):
		self.baseurlenabled = True
		self.mirrorlistenabled = False
		self.dirty = True

	def writeEnabled(self, tmpfile):
		if self.isEnabled():
			tmpfile.write("enabled=1\n")
		else:
			tmpfile.write("enabled=0\n")

	def writeVerifyEnabled(self, tmpfile):
		if self.isVerifyEnabled():
			tmpfile.write("gpgcheck=1\n")
		else:
			tmpfile.write("gpgcheck=0\n")

	def writeBaseURL(self, tmpfile):
		if not self.isBaseURLEnabled():
			tmpfile.write("#")

		tmpfile.write("baseurl=%s\n" % (self.getBaseURL(),))

	def writeMirrorList(self, tmpfile):
		if not self.isMirrorListEnabled():
			tmpfile.write("#")

		tmpfile.write("mirrorlist=%s\n" % (self.getMirrorList(),))

	def writeDescription(self, tmpfile):
		if self.getDescription() is not None:
			tmpfile.write("name=%s\n" % (self.getDescription(),))

	def write(self, stanzalines, tmpfile):
		if self.isRemoved():
			return

		# handle new repositories
		if stanzalines == [] and self.dirty is True:
			tmpfile.write("[%s]\n" % (self.getName(),))
			self.writeDescription(tmpfile)
			self.writeBaseURL(tmpfile)
			self.writeMirrorList(tmpfile)
			self.writeEnabled(tmpfile)
			self.writeVerifyEnabled(tmpfile)
			return

		if self.dirty is True:
			# handle existing repositories (preserve order)
			for l in stanzalines:
				if l[0:8] == 'enabled=':
					self.writeEnabled(tmpfile)
				elif l[0:9] == 'gpgcheck=':
					self.writeVerifyEnabled(tmpfile)
				elif l[0:8] == 'baseurl=' or l[0:9] == '#baseurl=':
					self.writeBaseURL(tmpfile)
				elif l[0:11] == 'mirrorlist=' or l[0:12] == '#mirrorlist=':
					self.writeMirrorList(tmpfile)
				else:
					tmpfile.write("%s\n" % (l,))
		else:
			for l in stanzalines:
				tmpfile.write("%s\n" % (l,))
